// Functions for exact event-driven simulation of IF models with exponential conductances
//
// See "Exact simulation of integrate-and-fire models with
// exponential synaptic conductances", R. Brette, Neural Computation (2006).
//
// R. Brette (brette@di.ens.fr)
// Feb 2006
//
exec('rho.sci');

SPIKE_COMPUTATION_ERROR = 1e-6;

// Parameters
// (these can be included in the state variable X in case all neurons don't share
// the same values)
//Vr = 0; // reset
//taus = 0.3; // relative to membrane time constant
//Ep = 5; // excitatory synaptic reversal potential (relative to threshold)
//Em = 0; // inhibitory synaptic reversal potential (relative to threshold)

// In  all functions:
// X(:,1) = V (membrane potential)
// X(:,2) = g
// X(:,3) = Es
// X(:,4) = time of last spike

function X=fUpdate_ExpCondIF(X,t)
  gt=X(:,2).*exp(-(t-X(:,4))/taus);
  X(:,1)=-taus*X(:,3).*gt.*rho(gt)+exp(X(:,4)-t+taus*(gt-X(:,2))).*(X(:,1)+taus*X(:,3).*X(:,2).*rho(X(:,2)));
  X(:,2)=gt;
  X(:,4)=t;
endfunction

function t=fNextSpike_ExpCondIF(X)
  V=X(1);
  g=X(2);
  Es=X(3);
  lastt=X(4);

  // Spike test
  if (Es<=1)
    t=-1;
  else
    gstar=1/(Es-1);
    if (g<=gstar)
      t=-1;
    else
      Y=fUpdate_ExpCondIF(X,lastt-taus*log(gstar/g));
      if (Y(1)<=1)
        t=-1;
      else
        // There is a spike for sure!
        t=lastt;
        while (1-V>SPIKE_COMPUTATION_ERROR)
        			t=t+(1-V)/(-V+g*(Es-V));
        			X=fUpdate_ExpCondIF(X,t);
        			V=X(1);
        			g=X(2);
        			Es=X(3);
         end
      end
    end
  end
endfunction

function X=fOutgoingSpike_ExpCondIF(X)
  X(:,1)=Vr;
endfunction

function X=fIncomingSpike_ExpCondIF(X,w,t)
  E=Ep*bool2s(w>0)+Em*bool2s(w<0);
  X(:,3)=(X(:,2).*X(:,3)+abs(w).*E)./(X(:,2)+abs(w));
  X(:,2)=X(:,2)+w;
endfunction

